"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.exportFunctions = void 0;
const glob_1 = require("glob");
/**
 * Exports all JS/TS Firebase Functions in all subdirectories
 * matching the filename pattern '{functionName}.function.js'.
 * @example
 * // Given the following:
 * src
 *   reactive
 *     onUserCreate.function.ts
 *     notAFunction.ts
 *   callable
 *     users
 *      getUser.function.ts
 *      deleteUser.function.ts
 *   emails
 *      sendEmail.function.ts
 *      Emailer.ts
 *
 * // These functions will be exported
 * ['onUserCreate', 'getUser', 'deleteUser', 'sendEmail']
 *
 */
function exportFunctions() {
    const filePaths = getMatchingFilepaths();
    const imports = getModules(filePaths);
    return getFirebaseFunctionsFromModules(imports);
}
exports.exportFunctions = exportFunctions;
function getMatchingFilepaths() {
    const pattern = `${process.cwd()}/**/*.function.js`;
    return (0, glob_1.sync)(pattern);
}
function getModules(filePaths) {
    return filePaths.map((path) => ({
        name: getFunctionNameFromFile(path),
        module: require(path),
    }));
}
function getFunctionNameFromFile(filePath) {
    var _a;
    return (_a = filePath.split("/").pop()) === null || _a === void 0 ? void 0 : _a.split(".function")[0];
}
function getFirebaseFunctionsFromModules(modules) {
    const functions = modules.map((module) => getFirebaseFunctionsFromExports(module)).flat();
    const exports = {};
    functions.forEach((fn) => {
        if (exports[fn.name]) {
            throw new Error(`Firebase Functions must have unique names. Please rename ${fn.name}.`);
        }
        exports[fn.name] = fn.fn;
    });
    return exports;
}
function getFirebaseFunctionsFromExports(imported) {
    const exported = imported.module;
    const exports = Object.keys(imported.module);
    return exports
        .filter((key) => isFirebaseFunction(exported[key]))
        .map((key) => ({
        name: key === "default" ? imported.name : key,
        fn: exported[key],
    }));
}
function isFirebaseFunction(exported) {
    return (typeof exported === "function" && (exported.name === "cloudFunction" || exported.name === ""));
}
